

package com.office.allreader.allofficefilereader.fc.hssf.formula.function;


import com.office.allreader.allofficefilereader.fc.hssf.formula.OperationEvaluationContext;
import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.ValueEval;



/**
 * For most Excel functions, involving references ((cell, area), (2d, 3d)), the references are
 * passed in as arguments, and the exact location remains fixed.  However, a select few Excel
 * functions have the ability to access cells that were not part of any reference passed as an
 * argument.<br/>
 * Two important functions with this feature are <b>INDIRECT</b> and <b>OFFSET</b><p/>
 *
 * When POI evaluates formulas, each reference argument is capable of evaluating any cell inside
 * its range.  Actually, even cells outside the reference range but on the same sheet can be
 * evaluated.  This allows <b>OFFSET</b> to be implemented like most other functions - taking only
 * the arguments, and source cell coordinates.
 *
 * For the moment this interface only exists to serve the <b>INDIRECT</b> which can decode
 * arbitrary text into cell references, and evaluate them..
 *
 * @author Josh Micich
 */
public interface FreeRefFunction {
	/**
	 * @param args the pre-evaluated arguments for this function. args is never <code>null</code>,
	 *             nor are any of its elements.
	 * @param ec primarily used to identify the source cell containing the formula being evaluated.
	 *             may also be used to dynamically create reference evals.
	 * @return never <code>null</code>. Possibly an instance of <tt>ErrorEval</tt> in the case of
	 * a specified Excel error (Exceptions are never thrown to represent Excel errors).
	 */
	ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec);
}
