package com.office.reader.viewer.activity

import android.annotation.SuppressLint
import android.app.Dialog
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.Paint
import android.graphics.pdf.PdfDocument
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import android.view.View
import android.view.WindowManager
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.FileProvider
import com.github.barteksc.pdfviewer.scroll.DefaultScrollHandle
import com.office.allreader.allofficefilereader.BuildConfig
import com.office.allreader.allofficefilereader.R
import com.office.reader.viewer.model.OnActionCallBack
import com.office.reader.viewer.util.Constants
import com.office.reader.viewer.util.Utils
import kotlinx.android.synthetic.main.activity_pdf_viewer.*
import kotlinx.android.synthetic.main.activity_screen_viewer.*
import kotlinx.android.synthetic.main.activity_screen_viewer.btnBack
import kotlinx.android.synthetic.main.activity_screen_viewer.pdfView
import kotlinx.android.synthetic.main.layout_dialog_gotopage.*
import java.io.File
import java.io.FileOutputStream
import java.io.IOException

class ScreenShotViewerActivity : AppCompatActivity() {

    private var outputPDF = ""
    private var pdfDocument: PdfDocument? = null
    private lateinit var mUrlImage: String
    private var mIsShowMenu = false
    private var titleBar: TextView? = null
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_screen_viewer)
        titleBar = findViewById<TextView>(R.id.title)
        initData()
        handleEvents()
    }

    private fun initData() {
        getExternalFilesDir(null)?.delete()
        outputPDF = getExternalFilesDir(null)?.absolutePath + "temp.pdf"
        mUrlImage = intent.getStringExtra(Constants.URL)!!
        titleBar!!.text = intent.getStringExtra(Constants.TITLE)
        var bitmap: Bitmap? = null

        try {
            if (this.mUrlImage.startsWith("http")) {
                Utils.downLoadFromNet(mUrlImage,object : OnActionCallBack<String> {
                    override fun onResponse(data: String) {
                        bitmap = MediaStore.Images.Media.getBitmap(
                                contentResolver,
                                Uri.fromFile(File(data))
                        )
                        makePDF(bitmap)
                    }
                })
            } else {
            bitmap = MediaStore.Images.Media.getBitmap(
                this.contentResolver,
                Uri.fromFile(File(mUrlImage))
            )
            makePDF(bitmap)
            }
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    private fun handleEvents() {
        btnBack.setOnClickListener {
            onBackPressed()
        }
    }

    @SuppressLint("NewApi")
    fun makePDF(bitmap: Bitmap?) {
        if(bitmap != null) {
            pdfDocument = PdfDocument()
            val pageInfo = PdfDocument.PageInfo.Builder(bitmap.width, bitmap.height, 1).create()
            val page: PdfDocument.Page = pdfDocument!!.startPage(pageInfo)
            val canvas = page.canvas
            val paint = Paint()
            paint.color = Color.parseColor("#FFFFFF")
            canvas.drawBitmap(bitmap, 0f, 0f, null)
            pdfDocument!!.finishPage(page)
            saveFile()
        }else{
            Toast.makeText(applicationContext, "File Load Error!", Toast.LENGTH_LONG).show()
        }
    }


    @SuppressLint("NewApi")
    fun saveFile() {
        if (pdfDocument == null) {
            Log.i("local-dev", "pdfDocument in 'saveFile' function is null")
            return
        }
        val root = File(
            Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS),
            "ImgToPDF"
        )
        var isDirectoryCreated = root.exists()
        if (!isDirectoryCreated) {
            isDirectoryCreated = root.mkdir()
        }
        val file = File(outputPDF)
        try {
            val fileOutputStream = FileOutputStream(file)
            pdfDocument!!.writeTo(fileOutputStream)
            pdfDocument!!.close()
            viewPDFFile()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }


    fun viewPDFFile() {
        pdfView.fromFile(File(outputPDF)).load()
    }

    override fun onBackPressed() {
        val returnIntent = Intent()
        setResult(RESULT_OK, returnIntent)
        finish()
    }
}