package com.office.reader.viewer.util

import android.annotation.SuppressLint
import android.app.Activity
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.provider.OpenableColumns
import java.lang.Exception
import android.graphics.Bitmap
import android.graphics.Bitmap.CompressFormat
import android.graphics.Canvas
import android.os.Environment
import android.provider.MediaStore
import android.text.TextUtils
import android.util.Log
import android.view.View
import android.widget.Toast
import com.office.reader.viewer.api.LoadFileModel
import com.office.reader.viewer.model.OnActionCallBack
import java.io.*
import java.util.HashSet
import com.simplemobiletools.commons.extensions.*
import com.simplemobiletools.commons.helpers.ensureBackgroundThread
import com.simplemobiletools.commons.helpers.isRPlus
import com.simplemobiletools.commons.models.FileDirItem
import com.stericson.RootShell.execution.Command
import com.stericson.RootTools.RootTools
import okhttp3.ResponseBody
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import android.os.StrictMode
import android.os.StrictMode.VmPolicy


object Utils {
    private const val TAG = "log | flutter_file_preview | "
    fun getFileName(uri: Uri, context: Context): String? {
        var result: String? = null
        if (uri.scheme != null && uri.scheme == "content") {
            try {
                context.contentResolver.query(uri, null, null, null, null).use { cursor ->
                    if (cursor != null && cursor.moveToFirst()) {
                        val indexDisplayName: Int =
                            cursor.getColumnIndex(OpenableColumns.DISPLAY_NAME)
                        if (indexDisplayName != -1) {
                            result = cursor.getString(indexDisplayName)
                        }
                    }
                }
            } catch (e: Exception) {

            }
        }
        if (result == null) {
            result = uri.lastPathSegment
        }
        return result
    }

    fun fileShareIntent(chooserTitle: String?, fileUri: Uri?): Intent? {
        val shareIntent = Intent(Intent.ACTION_SEND)
        shareIntent.putExtra(Intent.EXTRA_STREAM, fileUri)
        shareIntent.flags = Intent.FLAG_GRANT_READ_URI_PERMISSION
        shareIntent.type = "application/pdf"
        return Intent.createChooser(shareIntent, chooserTitle)
    }

    fun takeScreenshot(view: View): Bitmap? {
        val bitmap = Bitmap.createBitmap(view.width, view.height, Bitmap.Config.ARGB_8888)
        val canvas = Canvas(bitmap)
        view.draw(canvas)
        return bitmap
    }

    fun saveBitmap(bitmap: Bitmap, context: Context): String {
        val filePath: File =
            context.getExternalFilesDir(null)!!.absoluteFile
        val dir = File(filePath.absolutePath.toString() + "/AllDocument/")
        dir.mkdir()
        val nameFile = "screenshot.jpg"
        val file = File(dir, nameFile)
        try {
            val outputStream = FileOutputStream(file)
            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outputStream)
            outputStream.flush()
            outputStream.close()
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        } catch (e: IOException) {
            e.printStackTrace()
        } finally {
            return file.absolutePath
        }
    }

    fun saveBitmapByName(bitmap: Bitmap, context: Context, name: String): String {
        val filePath: File =
            context.getExternalFilesDir(null)!!.absoluteFile
        val dir = File(filePath.absolutePath.toString() + "/AllDocument/")
        dir.mkdir()
        val nameFile = "$name.jpg"
        val file = File(dir, nameFile)
        try {
            val outputStream = FileOutputStream(file)
            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outputStream)
            outputStream.flush()
            outputStream.close()
            Toast.makeText(
                context,
                "The screen shot stored in path that is " + file.absolutePath,
                Toast.LENGTH_LONG
            ).show()
        } catch (e: FileNotFoundException) {
            Toast.makeText(context, e.printStackTrace().toString(), Toast.LENGTH_LONG).show()
            e.printStackTrace()
        } catch (e: IOException) {
            Toast.makeText(context, e.printStackTrace().toString(), Toast.LENGTH_LONG).show()
            e.printStackTrace()
        } finally {
            return file.absolutePath
        }
    }

    fun saveBitmapTemp(bitmap: Bitmap, context: Context): String {
        val filePath: File =
            context.getExternalFilesDir(null)!!.absoluteFile
        val dir = File(filePath.absolutePath.toString() + "/AllDocumentTemp/")
        dir.mkdir()
        val nameFile = "screenshot.jpg"
        val file = File(dir, nameFile)
        try {
            val outputStream = FileOutputStream(file)
            bitmap.compress(Bitmap.CompressFormat.JPEG, 100, outputStream)
            outputStream.flush()
            outputStream.close()
        } catch (e: FileNotFoundException) {
            e.printStackTrace()
        } catch (e: IOException) {
            e.printStackTrace()
        } finally {
            return file.absolutePath
        }
    }

    @Throws(FileNotFoundException::class)
    fun writeBitmapToUri(
        context: Context,
        bitmap: Bitmap,
        uri: Uri?,
        compressFormat: CompressFormat?,
        i: Int
    ) {
        var outputStream: OutputStream? = null
        try {
            outputStream = context.contentResolver.openOutputStream(uri!!)
            bitmap.compress(compressFormat, i, outputStream)
        } finally {
            closeSafe(outputStream)
        }
    }

    private fun closeSafe(closeable: Closeable?) {
        if (closeable != null) {
            try {
                closeable.close()
            } catch (unused: IOException) {
            }
        }
    }

    private const val APP_DOC = "AppDoc"
    private const val KEY_PATH = "KeyPath"
    private const val KEY_PATH_RECENT = "KeyPathRecent"



    private fun runCommand(command: Command) {
        try {
            RootTools.getShell(true).add(command)
            Log.d("99999999", "Delete Success")
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun Activity.deleteFile(
        fileDirItem: FileDirItem,
        allowDeleteFolder: Boolean = false,
        callback: ((wasSuccess: Boolean) -> Unit)? = null
    ) {
        ensureBackgroundThread {
            deleteFileBg(fileDirItem, allowDeleteFolder, callback)
        }
    }

    fun deleteRecursively2(file: File): Boolean {
        if (file.isDirectory) {
            val files = file.listFiles() ?: return file.delete()
            for (child in files) {
                deleteRecursively2(child)
            }
        }

        return file.delete()
    }


    fun Activity.deleteFileBg(
        fileDirItem: FileDirItem,
        allowDeleteFolder: Boolean = false,
        callback: ((wasSuccess: Boolean) -> Unit)? = null
    ) {
        val path = fileDirItem.path
        if (isRestrictedSAFOnlyRoot(path)) {
            deleteAndroidSAFDirectory(path, allowDeleteFolder, callback)
        } else {
            val file = File(path)
            if (!isRPlus() && file.absolutePath.startsWith(internalStoragePath) && !file.canWrite()) {
                callback?.invoke(false)
                return
            }

            var fileDeleted =
                !isPathOnOTG(path) && ((!file.exists() && file.length() == 0L) || file.delete())
            if (fileDeleted) {
                deleteFromMediaStore(path) { needsRescan ->
                    if (needsRescan) {
                        rescanAndDeletePath(path) {
                            runOnUiThread {
                                callback?.invoke(true)
                            }
                        }
                    } else {
                        runOnUiThread {
                            callback?.invoke(true)
                        }
                    }
                }
            } else {
                if (getIsPathDirectory(file.absolutePath) && allowDeleteFolder) {
                    fileDeleted = deleteRecursively2(file)
                }

                if (!fileDeleted) {
                    if (needsStupidWritePermissions(path)) {

                    } else if (isRPlus()) {
                        val fileUris = getFileUrisFromFileDirItems(arrayListOf(fileDirItem)).second
                        deleteSDK30Uris(this, fileUris) { success ->
                            runOnUiThread {
                                callback?.invoke(success)
                            }
                        }
                    }
                }
            }
        }
    }

    fun deleteSDK30Uris(activity: Activity, uris: List<Uri>, callback: (success: Boolean) -> Unit) {
        if (isRPlus()) {
//            BaseSimpleActivity.funAfterDelete30File = callback
//            try {
//                val deleteRequest = MediaStore.createDeleteRequest(activity.contentResolver, uris).intentSender
//                startIntentSenderForResult(deleteRequest, DELETE_FILE_SDK_30_HANDLER, null, 0, 0, 0)
//            } catch (e: Exception) {
//               e.printStackTrace()
//            }
        } else {
            callback(false)
        }
    }

    fun updateSDK30Uris(uris: List<Uri>, callback: (success: Boolean) -> Unit) {
        callback(true)
    }


    fun Activity.renameFile(
        oldPath: String,
        newPath: String,
        isRenamingMultipleFiles: Boolean,
        callback: ((success: Boolean, useAndroid30Way: Boolean) -> Unit)? = null
    ) {
        val oldFile = File(oldPath)
        val newFile = File(newPath)
        val tempFile = try {
            createTempFile(oldFile) ?: return
        } catch (exception: Exception) {
            if (isRPlus() && exception is java.nio.file.FileSystemException) {
                // if we are renaming multiple files at once, we should give the Android 30+ permission dialog all uris together, not one by one
                if (isRenamingMultipleFiles) {
                    callback?.invoke(false, true)
                } else {
                    val fileUris =
                        getFileUrisFromFileDirItems(arrayListOf(File(oldPath).toFileDirItem(this))).second
                    updateSDK30Uris(fileUris) { success ->
                        if (success) {
                            val values = ContentValues().apply {
                                put(
                                    MediaStore.Images.Media.DISPLAY_NAME,
                                    newPath.getFilenameFromPath()
                                )
                            }

                            try {
                                contentResolver.update(fileUris.first(), values, null, null)
                                callback?.invoke(true, false)
                            } catch (e: Exception) {
                                showErrorToast(e)
                                callback?.invoke(false, false)
                            }
                        } else {
                            callback?.invoke(false, false)
                        }
                    }
                }
            } else {
                showErrorToast(exception)
                callback?.invoke(false, false)
            }
            return
        }

        val oldToTempSucceeds = oldFile.renameTo(tempFile)
        val tempToNewSucceeds = tempFile.renameTo(newFile)
        if (oldToTempSucceeds && tempToNewSucceeds) {
            if (newFile.isDirectory) {
                updateInMediaStore(oldPath, newPath)
                rescanPath(newPath) {
                    runOnUiThread {
                        callback?.invoke(true, false)
                    }
                    scanPathRecursively(newPath)
                }
            } else {
                if (!baseConfig.keepLastModified) {
                    newFile.setLastModified(System.currentTimeMillis())
                }
                updateInMediaStore(oldPath, newPath)
                scanPathsRecursively(arrayListOf(newPath)) {
                    runOnUiThread {
                        callback?.invoke(true, false)
                    }
                }
            }
        } else {
            tempFile.delete()
            runOnUiThread {
                callback?.invoke(false, false)
            }
        }
    }

    fun getEndFile(name: String): String {
        when {
            name.endsWith(".pdf") -> {
                return ".pdf"
            }
            name.endsWith(".doc") -> {
                return ".doc"
            }
            name.endsWith(".docx") -> {
                return ".docx"
            }
            name.endsWith(".xlsx") -> {
                return ".xlsx"
            }
            name.endsWith(".xls") -> {
                return ".xls"
            }
            name.endsWith(".pptx") -> {
                return ".pptx"
            }
            name.endsWith(".txt") -> {
                return ".txt"
            }
            name.endsWith(".ppt") -> {
                return ".ppt"
            }
            name.endsWith(".png") -> {
                return ".png"
            }
            name.endsWith(".jpg") -> {
                return ".jpg"
            }
            else -> return ""
        }
    }


    private fun getFileName(url: String): String? {
        return Md5Tool.hashKey(url).toString() + "." + getFileType(url)
    }

    @SuppressLint("LongLogTag")
    private fun getFileType(paramString: String): String {
        var str = ""
        if (TextUtils.isEmpty(paramString)) {
            Log.d(TAG, "paramString---->null")
            return str
        }
        Log.d(TAG, "paramString:$paramString")
        val i = paramString.lastIndexOf('.')
        if (i <= -1) {
            Log.d(TAG, "i <= -1")
            return str
        }
        str = paramString.substring(i + 1)
        Log.d(TAG, "paramString.substring(i + 1)------>$str")
        return str
    }

    private fun parseFormat(fileName: String): String? {
        return fileName.substring(fileName.lastIndexOf(".") + 1)
    }

    private fun parseName(url: String): String? {
        var fileName: String? = null
        try {
            fileName = url.substring(url.lastIndexOf("/") + 1)
        } finally {
            if (TextUtils.isEmpty(fileName)) {
                fileName = System.currentTimeMillis().toString()
            }
        }
        return fileName
    }

     fun getCacheDir(url: String): File {
        return File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM).absolutePath + "/file/")
    }

    @SuppressLint("LongLogTag")
     fun getCacheFile(url: String): File {
        val cacheFile = File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM).absolutePath + "/file/"
                + getFileName(url))
        Log.d(TAG, "Cache File = $cacheFile")
        return cacheFile
    }

    @JvmStatic
     fun downLoadFromNet(url: String, callbackData:  OnActionCallBack<String> ) {
        val cacheFile: File = getCacheFile(url)
        if (cacheFile.exists()) {
            if (cacheFile.length() <= 0) {
                cacheFile.delete()
                return
            }
        }
        LoadFileModel.loadPdfFile(url, object : Callback<ResponseBody> {
            @SuppressLint("LongLogTag")
            override fun onResponse(call: Call<ResponseBody>, response: Response<ResponseBody>) {
                val builder = VmPolicy.Builder()
                StrictMode.setVmPolicy(builder.build())
                var flag: Boolean
                var `is`: InputStream? = null
                val buf = ByteArray(2048)
                var len = 0
                var fos: FileOutputStream? = null
                try {
                    val responseBody = response.body()
                    `is` = responseBody.byteStream()
                    val total = responseBody.contentLength()
                    val file1: File = getCacheDir(url)
                    if (!file1.exists()) {
                        file1.mkdirs()
                    }
                    val fileN: File = getCacheFile(url)
                    Log.d(TAG, "File ： $fileN")
                    if (!fileN.exists()) {
                        val mkdir = fileN.createNewFile()
                    }
                    fos = FileOutputStream(fileN)
                    var sum: Long = 0
                    while (`is`.read(buf).also { len = it } != -1) {
                        fos.write(buf, 0, len)
                        sum += len.toLong()
                        val progress = (sum * 1.0f / total * 100).toInt()
                        Log.d(TAG, "File name:" + fileN.name + "Tien trinh: " + progress)
                    }
                    fos.flush()
                    callbackData.onResponse(fileN.absolutePath);
                } catch (e: Exception) {
                    Log.d(TAG, "Loi download file: $e")
                } finally {
                    try {
                        `is`?.close()
                    } catch (e: IOException) {
                    }
                    try {
                        fos?.close()
                    } catch (e: IOException) {
                    }
                }
            }

            @SuppressLint("LongLogTag")
            override fun onFailure(call: Call<ResponseBody>, t: Throwable) {
                Log.d(TAG, "That bai")
                Log.d(TAG,t.localizedMessage);
                val file: File = getCacheFile(url)
                if (!file.exists()) {
                    Log.d(TAG, "File khong ton tai")
                    file.delete()
                }
            }
        })
    }

}