package vn.itsol.filereader.file_reader;

import android.Manifest;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.provider.Settings;

import androidx.annotation.NonNull;

import com.office.reader.viewer.activity.OfficeViewerActivity;
import com.office.reader.viewer.activity.PdfViewerActivity;
import com.office.reader.viewer.activity.ScreenShotViewerActivity;
import com.office.reader.viewer.util.Constants;

import java.io.File;
import java.util.ArrayList;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import pub.devrel.easypermissions.BuildConfig;
import pub.devrel.easypermissions.EasyPermissions;

/** FileReaderPlugin */
public class FileReaderPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;
  private Context context;
  private Activity activity;


  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "file_reader");
    channel.setMethodCallHandler(this);
    context = flutterPluginBinding.getApplicationContext();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {

    switch (call.method) {
      case "openFile":
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
          if (!Environment.isExternalStorageManager()) {
            try
            {
              Uri uri = Uri.parse("package:" + context.getPackageName());
              System.out.println(uri.toString());
              Intent intent = new Intent(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION, uri);
              intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
              context.startActivity(intent);
            }
            catch (Exception ex)
            {
              Intent intent = new Intent();
              intent.setAction(Settings.ACTION_MANAGE_APP_ALL_FILES_ACCESS_PERMISSION);
              intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
              context.startActivity(intent);
            }
          } else {
              showFile(call);

          }
        } else {
        String[] perms = {Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE};

        String localSizePermission = call.argument("localSizePermission");
        if (!EasyPermissions.hasPermissions(context, perms)) {
          EasyPermissions.requestPermissions(activity, localSizePermission, 10086, perms);
        } else {
          showFile(call);
        }
        }
        result.success("done");
        break;
      default:
        result.notImplemented();
        break;
    }
  }

  private void showFile(@NonNull MethodCall call){
    String name = call.argument("path");
    String title = call.argument("title");
    String token = call.argument("token");
              Intent mIntent = null;
          if (name.endsWith(".pdf")) {
            mIntent = new Intent(context, PdfViewerActivity.class);
            mIntent.putExtra(Constants.URL, name);
          } else if (name.endsWith(".doc") || name.endsWith(".docx")) {
            mIntent = new Intent(context, OfficeViewerActivity.class);
            mIntent.putExtra(Constants.URL, name);
          } else if (name.endsWith(".xlsx") || name.endsWith(".xls")) {
            mIntent =  new Intent(context, OfficeViewerActivity.class);
            mIntent.putExtra(Constants.URL, name);
          } else if (name.endsWith(".pptx") || name.endsWith(".ppt")) {
            mIntent = new Intent(context, OfficeViewerActivity.class);
            mIntent.putExtra(Constants.URL, name);
          } else if (name.endsWith(".png") || name.endsWith(".jpg")) {
            mIntent = new Intent(context, ScreenShotViewerActivity.class);
            mIntent.putExtra(Constants.URL, name);
          } else if (name.endsWith(".txt")) {
            mIntent = new Intent(context, OfficeViewerActivity.class);
            mIntent.putExtra(Constants.URL, name);
          }

          mIntent.putExtra(Constants.TITLE, title);
          mIntent.putExtra(Constants.TOKEN, token);
          mIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
          context.startActivity(mIntent);
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    this.activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    this.activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivity() {

  }
}
